package com.hyphenate.chatdemo;

import android.content.Intent;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;

import androidx.appcompat.app.AppCompatActivity;

import com.hyphenate.EMCallBack;
import com.hyphenate.EMChatRoomChangeListener;
import com.hyphenate.EMMessageListener;
import com.hyphenate.EMValueCallBack;
import com.hyphenate.chat.EMChatRoom;
import com.hyphenate.chat.EMClient;
import com.hyphenate.chat.EMConversation;
import com.hyphenate.chat.EMCustomConversationFilter;
import com.hyphenate.chat.EMCustomMessageBody;
import com.hyphenate.chat.EMGroup;
import com.hyphenate.chat.EMMessage;
import com.hyphenate.chat.EMMessageBody;
import com.hyphenate.chat.EMMucSharedFile;
import com.hyphenate.chat.EMPushManager;
import com.hyphenate.chat.EMSilentModeParam;
import com.hyphenate.chat.EMSilentModeResult;
import com.hyphenate.chat.EMTextMessageBody;
import com.hyphenate.chatdemo.databinding.ActivityEmBinding;
import com.hyphenate.exceptions.HyphenateException;
import com.hyphenate.util.EMFileHelper;
import com.hyphenate.util.EMLog;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


public class EMActivity extends AppCompatActivity implements View.OnClickListener {
    private ActivityEmBinding mBinding;
    private static final String TAG = "EMActivity";
    private String conversationId = "250481334091777";
    //线上环境 easemob#easeim
//    private String chatRoomID = "248661699264519";
    //测试环境 easemob-demo#chatdemoui
    private String chatRoomID = "264325998641153";
    private String groupId = "243383590256648";

    private String messageId;
    private String username = "xu";
    private String to="pu";
    private EMChatRoomChangeListener chatRoomChangeListener;
    private String fileId="";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mBinding = ActivityEmBinding.inflate(getLayoutInflater());
        setContentView(mBinding.getRoot());
        initListener();
        initData();
    }

    private void initListener() {
        mBinding.btnSyncNoDisturb.setOnClickListener(this);
        mBinding.btnConversatonRemindType.setOnClickListener(this);
        mBinding.btnAddBlackList.setOnClickListener(this);
        mBinding.btnRemoveBlackList.setOnClickListener(this);
        mBinding.btnQueryBlackList.setOnClickListener(this);
        mBinding.btnAddContact.setOnClickListener(this);
        mBinding.btnRemoveContact.setOnClickListener(this);
        mBinding.btnAcceptInvitation.setOnClickListener(this);
        mBinding.btnRefuseInvitation.setOnClickListener(this);
        mBinding.btnQueryContacts.setOnClickListener(this);
        mBinding.btnQueryContactsFromServer.setOnClickListener(this);
        mBinding.btnQueryMsgCountWithDuration.setOnClickListener(this);
        mBinding.btnJoinChatroom.setOnClickListener(this);
        mBinding.btnLeaveChatroom.setOnClickListener(this);
        mBinding.btnGetUnreadMsgCount.setOnClickListener(this);
        mBinding.btnSetMsgParams.setOnClickListener(this);
        mBinding.btnSetFilterConversations.setOnClickListener(this);
        mBinding.btnSecondActivity.setOnClickListener(this);
        mBinding.btnQueryAllMsgs.setOnClickListener(this);
        mBinding.btnUploadGroupFile.setOnClickListener(this);
        mBinding.btnMakeCrash.setOnClickListener(this);
        mBinding.btnGroupFile.setOnClickListener(this);
        mBinding.btnDownloadGroupFile.setOnClickListener(this);
        mBinding.btnCustomMsg.setOnClickListener(this);
        mBinding.btnLoadMoreMsg.setOnClickListener(this);
        mBinding.btnChatroomCreatetime.setOnClickListener(this);
        mBinding.btnChatroomInWhitelist.setOnClickListener(this);
        mBinding.btnChatroomMuteExpireTime.setOnClickListener(this);
        mBinding.btnSendMessage.setOnClickListener(this);
        mBinding.btnModifyMessage.setOnClickListener(this);
        chatRoomChangeListener = new EMChatRoomChangeListener() {
            @Override
            public void onChatRoomDestroyed(String roomId, String roomName) {
                EMLog.e(TAG, "onChatRoomDestroyed roomId:" + roomId + " roomName:" + roomName);
            }

            @Override
            public void onMemberJoined(String roomId, String participant, String ext) {
                EMLog.e(TAG, "onMemberJoined roomId:" + roomId + " participant:" + participant + " ext:" + ext);
            }

            @Override
            public void onMemberExited(String roomId, String roomName, String participant) {
                EMLog.e(TAG, "onMemberExited roomId:" + roomId + " roomName:" + roomName + " participant:" + participant);
            }

            @Override
            public void onRemovedFromChatRoom(int reason, String roomId, String roomName, String participant) {
                EMLog.e(TAG, "onRemovedFromChatRoom reason:" + reason + " roomId:" + roomId + " roomName:" + roomName + " participant:" + participant);
            }

            @Override
            public void onMuteListAdded(String chatRoomId, List<String> mutes, long expireTime) {
                EMLog.e(TAG, "onMuteListAdded chatRoomId:" + chatRoomId + " mutes:" + mutes + " expireTime:" + expireTime);
            }

            @Override
            public void onMuteListRemoved(String chatRoomId, List<String> mutes) {
                EMLog.e(TAG, "onMuteListRemoved chatRoomId:" + chatRoomId + " mutes:" + mutes);
            }

            @Override
            public void onWhiteListAdded(String chatRoomId, List<String> whitelist) {

            }

            @Override
            public void onWhiteListRemoved(String chatRoomId, List<String> whitelist) {

            }

            @Override
            public void onAllMemberMuteStateChanged(String chatRoomId, boolean isMuted) {

            }

            @Override
            public void onAdminAdded(String chatRoomId, String admin) {

            }

            @Override
            public void onAdminRemoved(String chatRoomId, String admin) {

            }

            @Override
            public void onOwnerChanged(String chatRoomId, String newOwner, String oldOwner) {

            }

            @Override
            public void onAnnouncementChanged(String chatRoomId, String announcement) {

            }
        };
        EMClient.getInstance().chatroomManager().addChatRoomChangeListener(chatRoomChangeListener);
        mBinding.btnInput.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                username = s.toString();
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });

        EMClient.getInstance().chatManager().addMessageListener(new EMMessageListener() {
            @Override
            public void onMessageReceived(List<EMMessage> messages) {
                EMLog.e("xcp", "onMessageReceived");
                for (EMMessage message : messages) {
                    EMMessageBody body = message.getBody();
                    if(body instanceof EMCustomMessageBody){
                        EMLog.e("xcp", "onMessageReceived EMCustomMessageBody");
                        EMCustomMessageBody customMessageBody = (EMCustomMessageBody) body;
                        String event = customMessageBody.event();
                        EMLog.e("xcp", "event:" + event);
                    }else if(body instanceof EMTextMessageBody){
                        EMTextMessageBody messageBody = (EMTextMessageBody) (body);
                        String msg = messageBody.getMessage();
                        EMLog.e(TAG, "onMessageReceived onSuccess message:" + msg);
                    }
                    Map<String, Object> newExt = message.ext();
                    for (Map.Entry<String, Object> entry : newExt.entrySet()) {
                        EMLog.e(TAG, "onMessageReceived onSuccess key:" + entry.getKey() + " value:" + entry.getValue());
                    }

                }
            }

            @Override
            public void onMessageContentChanged(EMMessage messageModified, String operatorId, long operationTime) {
                Map<String, Object> newExt = messageModified.ext();
                for (Map.Entry<String, Object> entry : newExt.entrySet()) {
                    EMLog.e(TAG, "onMessageContentChanged onSuccess key:" + entry.getKey() + " value:" + entry.getValue());
                }
                EMMessageBody body = messageModified.getBody();
                if(body instanceof EMTextMessageBody){
                    EMTextMessageBody messageBody = (EMTextMessageBody) (body);
                    String message = messageBody.getMessage();
                    EMLog.e(TAG, "onMessageContentChanged onSuccess message:" + message);
                }

            }
        });
    }

    private void initData() {

    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_sync_no_disturb:
                syncNoDisturb();
                break;
            case R.id.btn_conversaton_remind_type:
                conversationRemindType();
                break;
            case R.id.btn_add_black_list:
                addBlacklist();
                break;
            case R.id.btn_remove_black_list:
                removeBlacklist();
                break;
            case R.id.btn_query_black_list:
                queryBlacklistLocal();
                break;
            case R.id.btn_add_contact:
                addContact();
                break;
            case R.id.btn_remove_contact:
                removeContact();
                break;
            case R.id.btn_accept_invitation:
                acceptInvitation();
                break;
            case R.id.btn_refuse_invitation:
                refuseInvitation();
                break;
            case R.id.btn_query_contacts:
                queryContacts();
                break;
            case R.id.btn_query_contacts_from_server:
                queryServerContacts();
                break;
            case R.id.btn_query_msg_count_with_duration:
                queryMsgCountWithDuration();
                break;
            case R.id.btn_join_chatroom:
                joinChatRoom();
                break;
            case R.id.btn_leave_chatroom:
                leaveChatRoom();
                break;
            case R.id.btn_get_unread_msg_count:
                getUnreadMessageCount();
                break;
            case R.id.btn_set_msg_params:
                setMsgParams();
                break;
            case R.id.btn_set_filter_conversations:
                filterConversations();
                break;
            case R.id.btn_second_activity:
                toSecondActivity();
                break;
            case R.id.btn_query_all_msgs:
                queryMsgsCount();
                break;
            case R.id.btn_upload_group_file:
                uploadGroupFile();
                break;
            case R.id.btn_group_file:
                scanGroupFile();
                break;
            case R.id.btn_download_group_file:
                downloadGroupFile();
                break;
            case R.id.btn_make_crash:
                makeCrash();
                break;
            case R.id.btn_custom_msg:
                sendCustomMsg();
                break;
            case R.id.btn_load_more_msg:
                loadMoreMsg();
                break;
            case R.id.btn_chatroom_createtime:
                chatroomCreateTime();
                break;
            case R.id.btn_chatroom_in_whitelist:
                chatroomInWhitelist();
                break;
            case R.id.btn_chatroom_mute_expire_time:
                chatroomMuteExpireTime();
                break;
            case R.id.btn_send_message:
                sendMessage();
                break;
            case R.id.btn_modify_message:
                modifyMessage();
                break;
        }
    }

    private void sendMessage() {
        EMMessage message = EMMessage.createTextSendMessage("hello", to);
        message.setAttribute("key1", "value1");
        message.setAttribute("key2", true);
        message.setAttribute("key3", 123);
        message.setMessageStatusCallback(new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.e(TAG, "sendMessage onSuccess");
                EMActivity.this.messageId=message.getMsgId();
                Map<String, Object> newExt = message.ext();
                for (Map.Entry<String, Object> entry : newExt.entrySet()) {
                    EMLog.e(TAG, "sendMessage onSuccess key:" + entry.getKey() + " value:" + entry.getValue());
                }

                EMTextMessageBody messageBody = (EMTextMessageBody) (message.getBody());
                String message = messageBody.getMessage();
                EMLog.e(TAG, "modifyMessage onSuccess message:" + message);
            }

            @Override
            public void onError(int code, String error) {
                EMLog.e(TAG, "sendMessage onError code:" + code + " error:" + error);
            }
        });
        EMClient.getInstance().chatManager().sendMessage(message);
    }

    private void modifyMessage(){
        EMTextMessageBody body = new EMTextMessageBody("你好");
        Map<String, Object> newExt = new HashMap<>();
        newExt.put("new key1", "new value1");
        newExt.put("new key2", 1);
        newExt.put("new key3", true);

//        EMClient.getInstance().chatManager().asyncModifyMessage(messageId, body,newExt, new EMValueCallBack<EMMessage>() {
//            @Override
//            public void onSuccess(EMMessage value) {
//                EMLog.e(TAG, "modifyMessage onSuccess");
//                Map<String, Object> newExt = value.ext();
//                for (Map.Entry<String, Object> entry : newExt.entrySet()) {
//                    EMLog.e(TAG, "modifyMessage onSuccess key:" + entry.getKey() + " value:" + entry.getValue());
//                }
//                EMTextMessageBody messageBody = (EMTextMessageBody) (value.getBody());
//                String message = messageBody.getMessage();
//                EMLog.e(TAG, "modifyMessage onSuccess message:" + message);
//            }
//
//            @Override
//            public void onError(int error, String errorMsg) {
//                EMLog.e(TAG, "modifyMessage onError error:" + error + " errorMsg:" + errorMsg);
//            }
//        });
    }

    private void leaveChatRoom() {
        EMClient.getInstance().chatroomManager().leaveChatRoom(chatRoomID, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "leaveChatRoom onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "leaveChatRoom onError code:" + code + " error:" + error);
            }
        });
    }

    private void chatroomMuteExpireTime() {
        EMChatRoom chatRoom = EMClient.getInstance().chatroomManager().getChatRoom(chatRoomID);
        if (chatRoom != null) {
            long muteExpireTimestamp = chatRoom.getMuteExpireTimestamp();
            EMLog.e(TAG, "chatroomMuteExpireTime muteExpireTimestamp:" + muteExpireTimestamp);
        }else{
            EMLog.e(TAG, "chatroomMuteExpireTime chatRoom is null");
        }
    }

    private void chatroomCreateTime() {
        EMChatRoom chatRoom = EMClient.getInstance().chatroomManager().getChatRoom(chatRoomID);
        if (chatRoom != null) {
            long createTimeStamp = chatRoom.getCreateTimestamp();
            EMLog.e(TAG, "chatroomCreateTime createTimeStamp:" + createTimeStamp);
        }else{
            EMLog.e(TAG, "chatroomCreateTime chatRoom is null");
        }
    }

    private void chatroomInWhitelist() {
        EMChatRoom chatRoom = EMClient.getInstance().chatroomManager().getChatRoom(chatRoomID);
        if (chatRoom != null) {
            boolean inWhitelist = chatRoom.isInWhitelist();
            EMLog.e(TAG, "chatroomInWhitelist inWhitelist:" + inWhitelist);
        }else{
            EMLog.e(TAG, "chatroomInWhitelist chatRoom is null");
        }
    }

    private void loadMoreMsg() {
        EMConversation conversation = EMClient.getInstance().chatManager().getConversation(to);
        if (conversation != null) {
            List<EMMessage> allMessages = conversation.loadMoreMsgFromDB("", 100);
            for (int i = 0; i < allMessages.size(); i++) {
                EMMessage emMessage = allMessages.get(i);
                EMLog.i(TAG, "loadMoreMsg emMessage:" + emMessage + "},isUnread:" + emMessage.isUnread());
            }
        }
    }

    private void sendCustomMsg() {
        EMMessage message = EMMessage.createSendMessage(EMMessage.Type.CUSTOM);
        EMCustomMessageBody body = new EMCustomMessageBody("temp");
        message.addBody(body);
        message.setTo("pu");
        message.setAttribute("isFaceShow", false);
        message.setAttribute("progress", 80);
        message.setAttribute("result", "success");
        message.setAttribute("had_received_video_msg_id", "123456");
        message.setAttribute("result", System.currentTimeMillis());

        message.setChatType(EMMessage.ChatType.Chat);
        EMClient.getInstance().chatManager().sendMessage(message);
        EMLog.e("xcp", "sendCmd event:"+body.event());
    }

    private void downloadGroupFile() {
        EMClient.getInstance().groupManager().asyncDownloadGroupSharedFile(groupId, fileId, getFilesDir().getAbsolutePath()+"/download/我是下载后的文件", new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "downloadGroupFile onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "downloadGroupFile onError code:" + code + " error:" + error);
            }
        });
    }

    private void scanGroupFile() {
        EMClient.getInstance().groupManager().asyncFetchGroupSharedFileList(groupId, 1, 10, new EMValueCallBack<List<EMMucSharedFile>>() {
            @Override
            public void onSuccess(List<EMMucSharedFile> value) {
                for (EMMucSharedFile emMucSharedFile : value) {
                    EMLog.i(TAG, "scanGroupFile onSuccess emMucSharedFile:name=" + emMucSharedFile.getFileName()+" ,fileupdateTime= "+emMucSharedFile.getFileUpdateTime()+" ,fileSize= "+emMucSharedFile.getFileSize()+" ,fileId= "+emMucSharedFile.getFileId());
                    fileId=emMucSharedFile.getFileId();
                }
            }

            @Override
            public void onError(int error, String errorMsg) {
                EMLog.i(TAG, "scanGroupFile onError error:" + error + " errorMsg:" + errorMsg);
            }
        });
    }

    private void makeCrash() {
        EMMessage textSendMessage = EMMessage.createTextSendMessage("hello",to);
        EMClient.getInstance().chatManager().sendMessage(textSendMessage);
    }

    private void uploadGroupFile() {
        EMGroup group = EMClient.getInstance().groupManager().getGroup(groupId);
        if (group == null) {
            EMLog.i(TAG, "uploadGroupFile group is null");
        }
        String filePath = getFilesDir().getAbsolutePath() + "/中文名文件2";
        boolean fileExist = EMFileHelper.getInstance().isFileExist(filePath);
        EMLog.e(TAG, "uploadGroupFile fileExist:" + fileExist);
        try {
            new Thread() {
                @Override
                public void run() {
                    super.run();
                    try {
                        EMClient.getInstance().groupManager().uploadGroupSharedFile(groupId, filePath, new EMCallBack() {
                            @Override
                            public void onSuccess() {
                                EMLog.i(TAG, "uploadGroupFile onSuccess");
                            }

                            @Override
                            public void onError(int code, String error) {
                                EMLog.i(TAG, "uploadGroupFile onError code:" + code + " error:" + error);
                            }
                        });
                    } catch (HyphenateException e) {

                        EMLog.i(TAG, "uploadGroupFile HyphenateException: errorMsg:" + e.getMessage());
                    }
                }
            }.start();
        } catch (Exception e) {
            EMLog.i(TAG, "uploadGroupFile HyphenateException: errorMsg:" + e.getMessage());
        }

    }

    private void queryMsgsCount() {
//        EMClient.getInstance().chatManager().asyncGetMessageCount(new EMValueCallBack<Integer>() {
//            @Override
//            public void onSuccess(Integer value) {
//                EMLog.e(TAG,"queryMsgsCount ="+ value);
//            }
//
//            @Override
//            public void onError(int error, String errorMsg) {
//                EMLog.i(TAG, "queryMsgsCount  error:" + error + " errorMsg:" + errorMsg);
//            }
//        });
    }

    private void toSecondActivity() {
        Intent intent = new Intent(this, SecondActivity.class);
        startActivity(intent);

//        int pageNum=1;
//        int pageSize=50;
//        EMClient.getInstance().chatroomManager().asyncFetchPublicChatRoomsFromServer(pageNum, pageSize, new EMValueCallBack<EMPageResult<EMChatRoom>>() {
//            @Override
//            public void onSuccess(EMPageResult<EMChatRoom> value) {
//
//            }
//
//            @Override
//            public void onError(int error, String errorMsg) {
//
//            }
//        });
//
//        EMClient.getInstance().chatroomManager().asyncFetchChatRoomFromServer(chatRoomID, new EMValueCallBack<EMChatRoom>() {
//            @Override
//            public void onSuccess(EMChatRoom value) {
//
//            }
//
//            @Override
//            public void onError(int error, String errorMsg) {
//
//            }
//        });
//        EMChatRoom chatRoom = EMClient.getInstance().chatroomManager().getChatRoom(chatRoomID);
    }

    private void filterConversations() {
        EMClient.getInstance().chatManager().asyncFilterConversationsFromDB(new EMCustomConversationFilter() {
            @Override
            public boolean filter(EMConversation emConversation) {
                EMLog.e(TAG, "begin-------filterConversations filter emConversation:" + emConversation);
                int allMsgCount = emConversation.getAllMsgCount();
                String id = emConversation.conversationId();
                EMConversation.EMConversationType type = emConversation.getType();
                int unreadMsgCount = emConversation.getUnreadMsgCount();
                EMLog.e(TAG, "end---------filterConversations filter allMsgCount:" + allMsgCount + " id:" + id + " type:" + type.ordinal() + " unreadMsgCount:" + unreadMsgCount);
                return true;
            }
        }, true, new EMValueCallBack<List<EMConversation>>() {
            @Override
            public void onSuccess(List<EMConversation> value) {
                EMLog.i(TAG, "filterConversations onSuccess value:" + value);
            }

            @Override
            public void onError(int error, String errorMsg) {
                EMLog.i(TAG, "filterConversations onError error:" + error + " errorMsg:" + errorMsg);
            }
        });
    }

    private void setMsgParams() {
        EMMessage message = EMMessage.createSendMessage(EMMessage.Type.CUSTOM);
        EMCustomMessageBody body = new EMCustomMessageBody("userCard");
        StringBuilder temp = new StringBuilder();
        for (int i = 0; i < 100; i++) {
            temp.append("aaaaaaaaaaaa");
        }
        for (int j = 0; j < 10; j++) {
            new Thread() {
                @Override
                public void run() {
                    for (int m = 0; m < 100; m++) {
                        Map<String, String> params = new HashMap<>();
                        for (int i = 0; i < 3000; i++) {
                            params.put("key" + i, "value" + i + temp.toString());
                        }
                        body.setParams(params);
                        EMLog.e(TAG, "setParams threadId=" + Thread.currentThread().getId());
                    }
                }
            }.start();
            new Thread() {
                @Override
                public void run() {
                    for (int i = 0; i < 400; i++) {
                        body.getParams();
                        EMLog.e(TAG, "getParams threadId=" + Thread.currentThread().getId());
                    }
                }
            }.start();
        }
        message.setBody(body);
        message.setTo("pu");
        message.setMessageStatusCallback(new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.e(TAG, "send message onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.e(TAG, "send message onError: " + code + " - " + error);
            }
        });
//        EMClient.getInstance().chatManager().sendMessage(message);
    }

    private void getUnreadMessageCount() {
        EMConversation conversation = EMClient.getInstance().chatManager().getConversation(username);
        if (conversation != null) {
            int unreadMsgCount = conversation.getUnreadMsgCount();
            EMLog.i(TAG, "getUnreadMessageCount unreadMsgCount:" + unreadMsgCount);
            List<EMMessage> allMessages = conversation.loadMoreMsgFromDB("", 100);
            for (int i = 0; i < allMessages.size(); i++) {
                EMMessage emMessage = allMessages.get(i);
                EMLog.i(TAG, "getUnreadMessageCount emMessage:" + emMessage + "},isUnread:" + emMessage.isUnread());
            }
        }
    }

    private void joinChatRoom() {
        String ext = "your ext info";
        boolean leaveOtherRooms = true;
        EMClient.getInstance().chatroomManager().joinChatRoom(chatRoomID, leaveOtherRooms, ext, new EMValueCallBack<com.hyphenate.chat.EMChatRoom>() {
            @Override
            public void onSuccess(com.hyphenate.chat.EMChatRoom value) {
                EMLog.i(TAG, "joinChatRoom onSuccess value:" + value);
            }

            @Override
            public void onError(int error, String errorMsg) {
                EMLog.i(TAG, "joinChatRoom onError error:" + error + " errorMsg:" + errorMsg);
            }
        });
    }

    private void queryMsgCountWithDuration() {
        String conversationId = "pu";
        EMConversation conversation = EMClient.getInstance().chatManager().getConversation(conversationId);
        if (conversation != null) {
            long startTimestamp = System.currentTimeMillis() - 24 * 60 * 60 * 1000;
            int count = conversation.getAllMsgCount(startTimestamp, System.currentTimeMillis());
            EMLog.i(TAG, "queryMsgCountWithDuration count:" + count);
        }
    }

    private void queryServerContacts() {
        EMClient.getInstance().contactManager().asyncGetAllContactsFromServer(new EMValueCallBack<List<String>>() {
            @Override
            public void onSuccess(List<String> value) {
                EMLog.i(TAG, "queryServerContacts onSuccess value:" + value);
            }

            @Override
            public void onError(int error, String errorMsg) {
                EMLog.i(TAG, "queryServerContacts onError error:" + error + " errorMsg:" + errorMsg);
            }
        });
    }

    private void queryContacts() {
//        EMClient.getInstance().contactManager().asyncFetchAllContactsFromLocal(new EMValueCallBack<List<EMContact>>() {
//            @Override
//            public void onSuccess(List<EMContact> value) {
//                EMLog.i(TAG, "queryContacts onSuccess value:" + value);
//            }
//
//            @Override
//            public void onError(int error, String errorMsg) {
//                EMLog.i(TAG, "queryContacts onError error:" + error + " errorMsg:" + errorMsg);
//            }
//        });
        try {
            List<String> contactsFromLocal = EMClient.getInstance().contactManager().getContactsFromLocal();
            EMLog.i(TAG, "queryContacts contactsFromLocal:" + contactsFromLocal);
        } catch (HyphenateException e) {
            throw new RuntimeException(e);
        }
    }

    private void refuseInvitation() {
        EMClient.getInstance().contactManager().asyncDeclineInvitation(username, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "refuseInvitation onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "refuseInvitation onError code:" + code + " error:" + error);
            }
        });
    }

    private void acceptInvitation() {
        EMClient.getInstance().contactManager().asyncAcceptInvitation(username, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "acceptInvitation onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "acceptInvitation onError code:" + code + " error:" + error);
            }
        });
    }

    private void removeContact() {
        EMClient.getInstance().contactManager().asyncDeleteContact(username, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "removeContact onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "removeContact onError code:" + code + " error:" + error);
            }
        });
    }

    private void addContact() {
        EMClient.getInstance().contactManager().asyncAddContact(username, "hello", new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "addContact onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "addContact onError code:" + code + " error:" + error);
            }
        });
    }

    private void queryBlacklistLocal() {
        List<String> blackListUsernames = EMClient.getInstance().contactManager().getBlackListUsernames();
        EMLog.i(TAG, "queryBlacklistLocal blackListUsernames:" + blackListUsernames);
    }

    private void removeBlacklist() {
        EMClient.getInstance().contactManager().asyncRemoveUserFromBlackList(username, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "removeBlacklist onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "removeBlacklist onError code:" + code + " error:" + error);
            }
        });
    }

    private void addBlacklist() {
        EMClient.getInstance().contactManager().asyncAddUserToBlackList(username, false, new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "addBlacklist onSuccess");
                queryBlacklistLocal();
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "addBlacklist onError code:" + code + " error:" + error);
            }
        });
    }


    private void conversationRemindType() {
        Map<String, EMConversation> allConversations = EMClient.getInstance().chatManager().getAllConversations();
        EMLog.e(TAG, "conversationRemindType allConversations:" + allConversations.size());

        String conversationId = "pu";
        EMSilentModeParam emSilentModeParam = new EMSilentModeParam(EMSilentModeParam.EMSilentModeParamType.REMIND_TYPE);
        //设置免打扰类型
        emSilentModeParam.setRemindType(EMPushManager.EMPushRemindType.NONE);
        EMClient.getInstance().pushManager().setSilentModeForConversation(conversationId, EMConversation.EMConversationType.Chat, emSilentModeParam, new EMValueCallBack<EMSilentModeResult>() {
            @Override
            public void onSuccess(EMSilentModeResult value) {
                EMLog.i(TAG, "conversationRemindType onSuccess value:" + value);
            }

            @Override
            public void onError(int error, String errorMsg) {
                EMLog.i(TAG, "conversationRemindType onError error:" + error + " errorMsg:" + errorMsg);
            }
        });

        EMConversation conversation = EMClient.getInstance().chatManager().getConversation(conversationId);
        if (conversation != null) {
            EMPushManager.EMPushRemindType emPushRemindType = conversation.pushRemindType();
            EMLog.i(TAG, "conversationRemindType emPushRemindType:" + emPushRemindType);
        }


    }

    private void syncNoDisturb() {
        EMClient.getInstance().pushManager().syncSilentModeConversationsFromServer(new EMCallBack() {
            @Override
            public void onSuccess() {
                EMLog.i(TAG, "syncNoDisturb onSuccess");
            }

            @Override
            public void onError(int code, String error) {
                EMLog.i(TAG, "syncNoDisturb onError code:" + code + " error:" + error);
            }
        });


    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        EMClient.getInstance().chatroomManager().removeChatRoomListener(chatRoomChangeListener);
    }
}